using System;
using System.Drawing;
using System.ComponentModel;
using System.Runtime.Serialization;
using System.Diagnostics;

using Syncfusion.Windows.Forms.Diagram;

namespace Syncfusion.Windows.Forms.Diagram.Samples.DynamicSymbol
{
	/// <summary>
	/// Custom Tool that draws a Rectangle with an bitmap node in it.
	/// </summary>
	public class CustomRectTool : Diagram.RectangleTool
	{
		/// <summary>
		/// Factory method used to create shape objects drawn by this tool.
		/// </summary>
		private ShapeFactory shapeFactory = null;

		protected Bitmap displaybmp;
		protected bool bDisplayImageAsNode = false;

		public Bitmap DisplayImage
		{
			get { return this.displaybmp; }
			set	{ this.displaybmp = value; }
		}

		public bool DisplayImageAsNode
		{
			get { return this.bDisplayImageAsNode; }
			set { this.bDisplayImageAsNode = value; }
		}

		public CustomRectTool() : base("CustomRectangleTool")
		{
			this.shapeFactory = new ShapeFactory(this.DefaultFactory);
		}
		
		public CustomRectTool(String name) : base(name)
		{			
			this.shapeFactory = new ShapeFactory(this.DefaultFactory);
		}

		// Create the Rectangle shape and insert the node using the InsertNodesCmd command. 
		// Depending on the 'bDisplayImageAsNode' flag the image is displayed inside the Shape 
		// as a fill texture or as a separate Bitmap node.
		protected override void OnRectangleComplete()
		{
			PointF[] worldPts = new PointF[2]
			{
				new PointF(this.rect.Left, this.rect.Top),
				new PointF(this.rect.Right, this.rect.Bottom)
			};

			if (this.shapeFactory != null)
			{
				Shape shape = this.shapeFactory(worldPts);
		
				if (shape != null)
				{
					InsertNodesCmd cmd = new InsertNodesCmd();
					cmd.Nodes.Add(shape);
					if(this.displaybmp != null)
					{						
						// Fill the Shape interior with the DisplayImage
						shape.SetPropertyValue("FillType", FillStyle.FillType.Texture);
						shape.SetPropertyValue("FillTexture", this.displaybmp);
						shape.SetPropertyValue("FillTextureWrapMode", System.Drawing.Drawing2D.WrapMode.Clamp);

						if(this.bDisplayImageAsNode == true)
						{
							Diagram.BitmapNode bmpnode = new BitmapNode(this.displaybmp, RectangleF.Inflate(this.rect, -5, -5));
							cmd.Nodes.Add(bmpnode);
						}
					}

					try
					{
						this.Controller.ExecuteCommand(cmd);
					}
					catch (Exception ex)
					{
						this.Controller.DeactivateTool(this);
						this.View.Refresh();
						throw ex;
					}
				}
			}

			this.Controller.DeactivateTool(this);
		}
	}
}
