#region Copyright Syncfusion Inc. 2001 -2004
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
//  Copyright Syncfusion Inc. All rights reserved. Use of this code is subject to the terms of our 
//  license. A copy of the current license can be obtained at any time by e-mailing licensing@syncfusion.com. 
//  Re-distribution in any form is strictly prohibited. Any infringement will be prosecuted under applicable laws. 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
#endregion 
 
using System;
using System.Drawing;
using System.ComponentModel;
using System.Runtime.Serialization;
using System.Diagnostics;

using Syncfusion.Windows.Forms.Diagram;

namespace Syncfusion.Windows.Forms.Diagram.Samples.DynamicSymbol
{
	[Serializable()]
	public class HostSymbol : Symbol
	{
		private Syncfusion.Windows.Forms.Diagram.Rectangle outerRect = null;
		
		public HostSymbol()
		{
			// Add an outer rectangle
			this.outerRect = new Syncfusion.Windows.Forms.Diagram.Rectangle(0, 0, 150, 100);
			this.outerRect.Name = "Rectangle";
			this.outerRect.FillStyle.Color = Color.Khaki;
			this.outerRect.EditStyle.Enabled = true;
			this.outerRect.EditStyle.AllowMove = true;
			this.outerRect.EditStyle.AllowResize = true;
			this.AppendChild(outerRect);

			MySymbol childsymbol = new MySymbol();
			this.AppendChild(childsymbol);
			childsymbol.Bounds = new RectangleF(10,10,childsymbol.Width,childsymbol.Height);
		}

		/// <summary>
		/// Serialization constructor for symbols.
		/// </summary>
		/// <param name="info">Serialization state information</param>
		/// <param name="context">Streaming context information</param>
		protected HostSymbol(SerializationInfo info, StreamingContext context) : base(info, context)
		{
		}		
		
		/// <summary>
		/// Populates a SerializationInfo with the data needed to
		/// serialize the target object.
		/// </summary>
		/// <param name="info">SerializationInfo object to populate</param>
		/// <param name="context">Destination streaming context</param>
		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);			
		}

		/// <summary>
		/// Called when deserialization is complete.
		/// </summary>
		/// <param name="sender">Object performing the deserialization</param>
		protected override void OnDeserialization(object sender)
		{
			base.OnDeserialization(sender);			
		}
	}
	
	/// <summary>
	/// Summary description for MySymbol.
	/// </summary>
	[Serializable()]
	public class MySymbol : Symbol
	{
		private Syncfusion.Windows.Forms.Diagram.Rectangle outerRect = null;
		private Ellipse innerEllipse = null;
		private int curEllipseColor = 0;
		private string strDescription = "Custom Symbol";

		static System.Drawing.Color[] ellipseColors =
		{
			Color.LightBlue,
			Color.Silver,
			Color.Yellow,
			Color.MidnightBlue
		};

		public MySymbol()
		{
			//////////////////////////////////////////////////////////////////
			// Add child nodes to the symbol programmatically
			//////////////////////////////////////////////////////////////////

			// Add an outer rectangle
			this.outerRect = new Syncfusion.Windows.Forms.Diagram.Rectangle(0, 0, 120, 80);
			this.outerRect.Name = "Rectangle";
			this.outerRect.FillStyle.Color = Color.Khaki;
			this.outerRect.EditStyle.Enabled = true;
			this.outerRect.EditStyle.AllowMove = true;
			this.outerRect.EditStyle.AllowResize = true;
			this.AppendChild(outerRect);

			// Add an inner ellipse
			this.innerEllipse = new Ellipse(10, 10, 100, 60);
			this.innerEllipse.Name = "Ellipse";
			this.innerEllipse.FillStyle.Color = ellipseColors[this.curEllipseColor];
			this.innerEllipse.EditStyle.Enabled = false;
			this.innerEllipse.EditStyle.AllowMove = true;
			this.innerEllipse.EditStyle.AllowResize = true;
			this.AppendChild(innerEllipse);

			Label lbl = this.AddLabel("Hello world", BoxPosition.Center);
			lbl.BackgroundStyle.Color = Color.Transparent;
		}

		/// <summary>
		/// Serialization constructor for symbols.
		/// </summary>
		/// <param name="info">Serialization state information</param>
		/// <param name="context">Streaming context information</param>
		protected MySymbol(SerializationInfo info, StreamingContext context) : base(info, context)
		{
			this.strDescription = info.GetString("strDescription");
		}

		/// <summary>
		/// This property provides an example of adding a member variable to a custom symbol
		/// and demonstrates how to handle serialization for this property.
		/// </summary>
		[
		Browsable(true),
		Category("General"),
		Description("Description of the symbol"),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)
		]
		public string Description
		{
			get
			{
				return this.strDescription;
			}
			set
			{
				this.strDescription = value;
			}
		}

		protected override void OnMouseEnter(NodeMouseEventArgs evtArgs)
		{
			this.outerRect.FillStyle.Color = Color.Green;

			Trace.WriteLine(evtArgs.Node.Name);
		}

		protected override void OnMouseLeave(NodeMouseEventArgs evtArgs)
		{
			this.outerRect.FillStyle.Color = Color.Khaki;
		}

		protected override void OnClick(NodeMouseEventArgs evtArgs)
		{
			if (this.curEllipseColor == ellipseColors.Length-1)
			{
				this.curEllipseColor = 0;
			}
			else
			{
				this.curEllipseColor++;
			}
			this.innerEllipse.FillStyle.Color = ellipseColors[this.curEllipseColor];
		}

		/// <summary>
		/// Populates a SerializationInfo with the data needed to
		/// serialize the target object.
		/// </summary>
		/// <param name="info">SerializationInfo object to populate</param>
		/// <param name="context">Destination streaming context</param>
		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);

			// Additional member variables are serialized here
			info.AddValue("strDescription", this.strDescription);
		}

		/// <summary>
		/// Called when deserialization is complete.
		/// </summary>
		/// <param name="sender">Object performing the deserialization</param>
		protected override void OnDeserialization(object sender)
		{
			base.OnDeserialization(sender);

			// Get references to the child objects
			this.outerRect = this.GetChildByName("Rectangle") as Syncfusion.Windows.Forms.Diagram.Rectangle;
			this.innerEllipse = this.GetChildByName("Ellipse") as Ellipse;

			// Determine the current ellipse color
			if (this.innerEllipse != null)
			{
				System.Drawing.Color ellipseColor = this.innerEllipse.FillStyle.Color;
				for (int colorIdx = 0; colorIdx < ellipseColors.Length; colorIdx++)
				{
					if (ellipseColors[colorIdx] == ellipseColor)
					{
						this.curEllipseColor = colorIdx;
						break;
					}
				}
			}
		}
	}
}
